@@ -20,6 +20,7 @@ import hicstuff.iteralign as hci
 import hicstuff.filter as hcf
 import hicstuff.io as hio
 import hicstuff.distance_law as hcdl
+import hicstuff.cutsite as hcc
 import matplotlib
 import pathlib
 from hicstuff.version import __version__
@@ -157,7 +158,11 @@ def bam2pairs(bam1, bam2, out_pairs, info_contigs, min_qual=30):
         # Remember if all reads in one bam file have been read
         exhausted = [False, False]
         # Iterate on both BAM simultaneously
+        end_regex = re.compile(r'/[12]$')
         for end1, end2 in itertools.zip_longest(forward, reverse):
+            # Remove end-specific suffix if any
+            end1.query_name = re.sub(end_regex, '', end1.query_name)
+            end2.query_name = re.sub(end_regex, '', end2.query_name)
             # Both file still have reads
             # Check if reads pass filter
             try:
@@ -461,7 +466,7 @@ def full_pipeline(
     enzyme=5000,
     filter_events=False,
     force=False,
-    iterative=False,
+    mapping="normal",
     mat_fmt="graal",
     min_qual=30,
     min_size=0,
@@ -491,10 +496,7 @@ def full_pipeline(
     input2 : str
         Path to the Hi-C reads in fastq format (forward), the aligned Hi-C reads
         in BAM format, or None, depending on the value of start_stage.
-    enzyme : int or str
-        Name of the enzyme used for the digestion (e.g "DpnII"). If an integer
-        is used instead, the fragment attribution will be done directly using a
-        fixed chunk size.
+    enzyme : int or strtest_data/genome/seq.fa
     circular : bool
         Use if the genome is circular.
     out_dir : str or None
@@ -513,9 +515,11 @@ def full_pipeline(
         Number of threads to use for parallel operations.
     no_cleanup : bool
         Whether temporary files should be deleted at the end of the pipeline.
-    iterative : bool
-        Use iterative mapping. Truncates and extends reads until unambiguous
-        alignment.
+    mapping : str
+        normal|iterative|cutsite. Use normal, iterative or cutsite mapping. 
+        "normal": Normal alignement. "iterative": Truncates and extends reads 
+        until unambiguous alignment. "cutsite": Digest reads at religation sites 
+        and build new pairs from the fragments created.
     filter_events : bool
         Filter spurious or uninformative 3C events. Requires a restriction enzyme.
     force : bool
@@ -754,6 +758,38 @@ def full_pipeline(
     # Detect if multiple enzymes are given
     if re.search(",", enzyme):
         enzyme = enzyme.split(",")
+        
+    # Define mapping choice (default normal):
+    if mapping == "normal":
+        iterative = False
+    elif mapping == "iterative":
+        iterative = True   
+    elif mapping == "cutsite":
+        # If no enzyme given use iterative alignment.
+        try:
+            int(enzyme)
+            logger.warning("No enzyme has been given. Can't map using cutsite, iterative mapping will be used instead.")
+            iterative = True
+        # If cutsite enabled and enzyme given, cut the reads before making a 
+        # normal alignment.
+        except ValueError:
+            itertative = False
+            digest_for = _tmp_file("digest_for.fq.gz")
+            digest_rev = _tmp_file("digest_rev.fq.gz")
+            hcc.cut_ligation_sites(
+                fq_for=reads1,
+                fq_rev=reads2,
+                digest_for=digest_for,
+                digest_rev=digest_rev,
+                enzyme=enzyme,
+                mode="for_vs_rev",
+                n_cpu=threads,
+            )
+            reads1, reads2 = digest_for, digest_rev
+    else:
+        logger.error("mapping must be either normal, iterative or cutsite.")
+        raise ValueError
+       
     # Perform genome alignment
     if start_stage == 0:
         align_reads(
